﻿<#
.SYNOPSIS
  Queries Lync Server 2010/2013 pools and Skype for Business 2015 pools and retrieves what Cumulative Update is installed on each server in each pool.

.DESCRIPTION
  Queries Lync Server 2010/2013 pools and Skype for Business 2015 pools and retrieves what Cumulative Update is installed on each server in each pool.

.NOTES
  Version               : 3.4 - See changelog at http://www.ucunleashed.com/2642
  Wish list             : Better error trapping
  Rights Required       : CSAdministrator and local admin rights on the target computers
  Sched Task Required   : No
  Lync Server Version   : 2010 and 2013, and Skype for Business Server 2015
  Author(s)             : Dave Howe
                        : Pat Richard
  Email/Blog/Twitter    : daveh@microsoft.com
                          pat@innervation.com   http://www.ucunleashed.com @patrichard
  Dedicated Post        : http://www.ucunleashed.com/2641
  Disclaimer            : You running this script means you won't blame me if this breaks your stuff. This script is provided AS IS
                          without warranty of any kind. I disclaim all implied warranties including, without limitation, any implied
                          warranties of merchantability or of fitness for a particular purpose. The entire risk arising out of the use
                          or performance of the sample scripts and documentation remains with you. In no event shall I be liable for
                          any damages whatsoever (including, without limitation, damages for loss of business profits, business
                          interruption, loss of business information, or other pecuniary loss) arising out of the use of or inability
                          to use the script or documentation.
  Acknowledgements      :
  Assumptions           : ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations           : PSRemoting must be enabled on the servers that are queried. It's enabled by default on Windows Server 2012
                          and later. For information on enabling PSRemoting, see http://technet.microsoft.com/en-us/library/hh849694.aspx
  Known issues          : Edge servers that aren't domain joined will show as "PSRemoting Failure".

.EXAMPLE
  .\Get-CsUpdateVersion.ps1

  Description
  -----------
  Runs the script and queries all pools in the environment.

.EXAMPLE
  .\Get-CsUpdateVersion.ps1 -PoolFqdn pool.contoso.com

  Description
  -----------
  Runs the script and queries just the pool specified.
#>
#Requires -Version 3.0

[CmdletBinding(SupportsShouldProcess)]
param (
  [string] $TargetFolder = "$env:SystemDrive\_Install",
  
  # Specifies a single pool to query
  [parameter(ValueFromPipeline, ValueFromPipelineByPropertyName)]
  [string] $PoolFqdn,

  # Skips the check for script age
  [parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
  [switch] $SkipUpdateCheck
)
[string] $Version = "3.4"
[string] $LogPath = "$TargetFolder\logs\$env:ComputerName" + " {0:yyyy-MM-dd hh-mmtt}.log" -f (Get-Date)
[string] $LogDivider = "------------------------------"

#region functions

function Get-UpdateInfo {
<#
.SYNOPSIS
  Queries an online XML source for version information to determine if a new version of the script is available.

.DESCRIPTION
  Queries an online XML source for version information to determine if a new version of the script is available.

.NOTES
  Version               : 1.0 - See changelog at http://www.ehloworld.com/3168 for fixes & changes introduced with each version
  Wish list             : Better error trapping
  Rights Required       : N/A
  Sched Task Required   : No
  Lync/Skype4B Version  : N/A
  Author/Copyright      : © Pat Richard, Office Servers and Services (Skype for Business) MVP - All Rights Reserved
  Email/Blog/Twitter    : pat@innervation.com  http://www.ehloworld.com  @patrichard
  Dedicated Post        : http://www.ehloworld.com/3168
  Disclaimer            : You running this script means you won't blame me if this breaks your stuff. This script is
                          provided AS IS without warranty of any kind. I disclaim all implied warranties including,
                          without limitation, any implied warranties of merchantability or of fitness for a particular
                          purpose. The entire risk arising out of the use or performance of the sample scripts and
                          documentation remains with you. In no event shall I be liable for any damages whatsoever
                          (including, without limitation, damages for loss of business profits, business interruption,
                          loss of business information, or other pecuniary loss) arising out of the use of or inability
                          to use the script or documentation.
  Acknowledgements      : Reading XML files 
                          http://stackoverflow.com/questions/18509358/how-to-read-xml-in-powershell
                          http://stackoverflow.com/questions/20433932/determine-xml-node-exists
  Assumptions           : ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations           : 
  Known issues          : 

.LINK
  http://www.ehloworld.com/3168

.EXAMPLE
  Get-UpdateInfo -Article 123

  Description
  -----------
  Runs function to check for updates to article/script 123.

.INPUTS
  None. You cannot pipe objects to this script.
#>
  [CmdletBinding(SupportsShouldProcess, SupportsPaging)]
  param (
    [string] $article
  )
  [bool] $HasInternetAccess = ([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]'{DCB00C01-570F-4A9B-8D69-199FDBA5723B}')).IsConnectedToInternet)
  if ($HasInternetAccess){
    [xml] $xml = (New-Object System.Net.WebClient).DownloadString("http://www.ehloworld.com/downloads/version.xml")
    [string] $Ga = ($xml.catalog.article | Where-Object {$_.id -eq $article}).version
    if (($xml.catalog.article | Where-Object {$_.id -eq $article}).versionInfo){
      [string] $changelog = "This version includes: " + ($xml.catalog.article | Where-Object {$_.id -eq $article}).versionInfo
    }
    if ($Ga -gt $version){
      Write-Log -Level Warn -Message "Outdated version. Version $Ga is latest version. Prompting user to upgrade." -NoConsole
      if ((New-Popup -Message "Version $Ga is now available. $changelog `n`nWould you like to download it?" -Title "New version available!" -Buttons YesNo -Icon Question) -eq 6){
        Write-Log -Message "User elected to download latest version." -Indent 1 -NoConsole
        Write-Log -Message "Opening Internet Explorer and going to http://www.ehloworld.com/$article#downloads." -NoConsole -Indent 1
        Start-Process "http://www.ehloworld.com/$article#downloads"
        Write-Log -Level Warn -Message "Script is exiting. Please start the new version of the script once you've downloaded it." -NoLog
        Exit
      }else{
        Write-Log -Level Warn -Message "User elected NOT to download latest version." -Indent 1 -NoConsole
      }
    }elseif ($Ga -eq $Version){
      Write-Log -Message "Current version. Version $Ga is latest version." -NoConsole
    }else{
      Write-Log -Level Warn -Message "Possibly regressed version. Version $Ga is latest version." -NoConsole
    }
  }else{
    Write-Log -Level Warn -Message "Unable to check online for update info." -NoConsole
  }
} # end function function Get-UpdateInfo

function New-Popup {
  <#
  .Synopsis
    Display a Popup Message

  .Description
    This command uses the Wscript.Shell PopUp method to display a graphical message
    box. You can customize its appearance of icons and buttons. By default the user
    must click a button to dismiss but you can set a timeout value in seconds to
    automatically dismiss the popup.

    The command will write the return value of the clicked button to the pipeline:
      OK     = 1
      Cancel = 2
      Abort  = 3
      Retry  = 4
      Ignore = 5
      Yes    = 6
      No     = 7

    If no button is clicked, the return value is -1.

  .Example
  PS C:\> new-popup -message "The update script has completed" -title "Finished" -time 5

  This will display a popup message using the default OK button and default
  Information icon. The popup will automatically dismiss after 5 seconds.

  .Notes
    Last Updated: April 8, 2013
    Version     : 1.0

  .Inputs
    None

  .Outputs
    integer

    Null   = -1
    OK     = 1
    Cancel = 2
    Abort  = 3
    Retry  = 4
    Ignore = 5
    Yes    = 6
    No     = 7
  #>
  [CmdletBinding(SupportsShouldProcess, SupportsPaging)]
  Param (
    # Message to be displayed in the popup message dialog box.
    [Parameter(Position = 0, Mandatory = $True, HelpMessage = "Enter a message for the popup")]
    [ValidateNotNullorEmpty()]
    [string] $Message,

    # Title of the popup dialog box.
    [Parameter(Position = 1, Mandatory = $True, HelpMessage = "Enter a title for the popup")]
    [ValidateNotNullorEmpty()]
    [string] $Title,

    # If defined, automatically dismisses the dialog box after the specified number of seconds. Defaults to 0, which requires a mouse click.
    [Parameter(Position = 2, HelpMessage = "How many seconds to display? Use 0 require a button click.")]
    [ValidateScript({$_ -ge 0})]
    [int] $Time = 0,

    # Type of buttons to display.
    [Parameter(Position = 3, HelpMessage = "Enter a button group")]
    [ValidateNotNullorEmpty()]
    [ValidateSet("OK","OKCancel","AbortRetryIgnore","YesNo","YesNoCancel","RetryCancel")]
    [string] $Buttons = "OK",

    # Icon set to use in dialog box.
    [Parameter(Position = 4, HelpMessage = "Enter an icon set")]
    [ValidateNotNullorEmpty()]
    [ValidateSet("Stop","Question","Exclamation","Information" )]
    [string] $Icon = "Information"
  ) # end of param block

  #convert buttons to their integer equivalents
  Switch ($Buttons) {
    "OK"               {$ButtonValue = 0}
    "OKCancel"         {$ButtonValue = 1}
    "AbortRetryIgnore" {$ButtonValue = 2}
    "YesNo"            {$ButtonValue = 4}
    "YesNoCancel"      {$ButtonValue = 3}
    "RetryCancel"      {$ButtonValue = 5}
  }

  #set an integer value for Icon type
  Switch ($Icon) {
    "Stop"        {$iconValue = 16}
    "Question"    {$iconValue = 32}
    "Exclamation" {$iconValue = 48}
    "Information" {$iconValue = 64}
  }

  #create the COM Object
  Try {
    $wshell = New-Object -ComObject Wscript.Shell -ErrorAction Stop
    #Button and icon type values are added together to create an integer value
    $wshell.Popup($Message,$Time,$Title,$ButtonValue+$iconValue)
  }

  Catch {
    #You should never really run into an exception in normal usage
    Write-Log -Level Warn -Message "Failed to create Wscript.Shell COM object"
    Write-Log -Level Warn -Message $_.exception.message
  }
} # end function New-Popup

function Set-ModuleStatus {
  [CmdletBinding(SupportsShouldProcess)]
  param  (
    [parameter(ValueFromPipeline, ValueFromPipelineByPropertyName, Mandatory, HelpMessage = "No module name specified!")]
    [ValidateNotNullOrEmpty()]
    [string] $name
  )
  PROCESS{
    # Executes once for each pipeline object
    # the $_ variable represents the current input object
    if (!(Get-Module -name "$name")) {
      if (Get-Module -ListAvailable | Where-Object Name -eq "$name") {
        Import-Module -Name "$name"
        # module was imported
        # return $true
      } else {
        # module was not available
        # return $false
      }
    } else {
      # Write-Output "$_ module already imported"
      # return $true
    }
  } # end PROCESS
} # end function Set-ModuleStatus

function Write-Log {
  <#
  .SYNOPSIS
    Extensive function to write data to either the console screen, a log file, and/or a Windows event log.

  .DESCRIPTION
    Extensive function to write data to either the console screen, a log file, and/or a Windows event log. Data can be written as info, warning, error, and includes indentation, time stamps, etc.

  .NOTES
    Version               : 2.8
    Wish list             : Better error trapping
    Rights Required       : Local administrator on server
    Sched Task Required   : No
    Lync/Skype4B Version  : N/A
    Author/Copyright      :
    Email/Blog/Twitter    : pat@innervation.com  http://www.ehloworld.com  @patrichard
    Dedicated Post        :
    Disclaimer            : You running this script means you won't blame me if this breaks your stuff. This script is
                            provided AS IS without warranty of any kind. I disclaim all implied warranties including,
                            without limitation, any implied warranties of merchantability or of fitness for a particular
                            purpose. The entire risk arising out of the use or performance of the sample scripts and
                            documentation remains with you. In no event shall I be liable for any damages whatsoever
                            (including, without limitation, damages for loss of business profits, business interruption,
                            loss of business information, or other pecuniary loss) arising out of the use of or inability
                            to use the script or documentation.
    Acknowledgements      : Test for log names and sources
                            http://powershell.com/cs/blogs/tips/archive/2013/06/10/testing-event-log-names-and-sources.aspx
    Assumptions           : ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
    Limitations           :
    Known issues          :

  .LINK


  .EXAMPLE
    .\

    Description
    -----------


  .INPUTS
    None. You cannot pipe objects to this script.
  #>
  [CmdletBinding(SupportsShouldProcess, SupportsPaging)]
  param(
    # The type of message to be logged. Alias is 'type'.
    [Parameter(Position = 0, ValueFromPipeline = $true, ValueFromPipelineByPropertyName)]
    [ValidateSet("Error", "Warn", "Info")]
    [ValidateNotNullOrEmpty()]
    [string] $level = "Info",

    # The message to be logged.
    [Parameter(Position = 1, ValueFromPipeline = $true, ValueFromPipelineByPropertyName, Mandatory = $true, HelpMessage = "No message specified.")]
    [ValidateNotNullOrEmpty()]
    [string] $Message,

    # Specifies that $message should not the sent to the log file.
    [Parameter(Position = 2, ValueFromPipelineByPropertyName)]
    [switch] $NoLog,

    # Specifies to not display the message to the console.
    [Parameter(Position = 3, ValueFromPipelineByPropertyName)]
    [switch] $NoConsole,

    # The number of spaces to indent the message in the log file.
    [Parameter(Position = 4, ValueFromPipelineByPropertyName)]
    [ValidateRange(1,30)]
    [ValidateNotNullOrEmpty()]
    [Int16] $Indent = 0,

    # Specifies what color the text should be be displayed on the console. Ignored when switch 'NoConsoleOut' is specified.
    [Parameter(Position = 5, ValueFromPipelineByPropertyName)]
    [ValidateSet("Black", "DarkMagenta", "DarkRed", "DarkBlue", "DarkGreen", "DarkCyan", "DarkYellow", "Red", "Blue", "Green", "Cyan", "Magenta", "Yellow", "DarkGray", "Gray", "White")]
    [ValidateNotNullOrEmpty()]
    [String] $ConsoleForeground = 'White',

    # Existing log file is deleted when this is specified. Alias is 'Overwrite'.
    [Parameter(Position = 6, ValueFromPipelineByPropertyName)]
    [Switch] $Clobber,

    # The name of the system event log, e.g. 'Application'. Note that writing to the system event log requires elevated permissions.
    [Parameter(Position = 7, ValueFromPipelineByPropertyName)]
    [ValidateSet("Application","System","Security","Lync Server","Microsoft Office Web Apps")]
    [ValidateNotNullOrEmpty()]
    [String] $EventLogName,

    # The name to appear as the source attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
    [Parameter(Position = 8, ValueFromPipelineByPropertyName)]
    [ValidateNotNullOrEmpty()]
    [String] $EventSource = $($MyInvocation.ScriptName).Name,

    # The ID to appear as the event ID attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
    [Parameter(Position = 9, ValueFromPipelineByPropertyName)]
    [ValidateRange(1,65535)]
    [ValidateNotNullOrEmpty()]
    [Int32] $EventID = 1,

    # The text encoding for the log file. Default is ASCII.
    [Parameter(Position = 10, ValueFromPipelineByPropertyName)]
    [ValidateSet("Unicode","Byte","BigEndianUnicode","UTF8","UTF7","UTF32","ASCII","Default","OEM")]
    [ValidateNotNullOrEmpty()]
    [String] $LogEncoding = "ASCII"
  ) # end of param block
  try {
    [string]$LogFolder = Split-Path $LogPath -Parent
    if (-not(Test-Path $LogFolder)){New-Item $LogFolder -type Directory | Out-Null}
    $msg = "{0} : {1} : {2}{3}" -f (Get-Date -Format "yyyy-MM-dd HH:mm:ss"), $Level.ToUpper(), ("  " * $Indent), $Message
    if (-not($NoConsole)){
      switch ($level) {
        "Error" {$Host.UI.WriteErrorLine("$Message")}
        "Warn" {Write-Warning $Message}
        "Info" {Write-Host $Message -ForegroundColor $ConsoleForeground}
      }
    }
    if (-not($NoLog)){
      if ($Clobber) {
        $msg | Out-File -FilePath $LogPath -Encoding $LogEncoding -Force
      } else {
        $msg | Out-File -FilePath $LogPath -Encoding $LogEncoding -Append
      }
    }

    # http://social.technet.microsoft.com/Forums/en-US/winserverpowershell/thread/e172f039-ce88-4c9f-b19a-0dd6dc568fa0/

    if ($EventLogName) {
      if (-not $EventSource) {
        [string] $EventSource = $([IO.FileInfo] $MyInvocation.ScriptName).Name
      }

      if(-not [Diagnostics.EventLog]::SourceExists($EventSource)) {
        [Diagnostics.EventLog]::CreateEventSource($EventSource, $EventLogName)
      }

      switch ($Level) {
        "Error" {$EntryType = "Error"}
        "Warn"  {$EntryType = "Warning"}
        "Info"  {$EntryType = "Information"}
        Default  {$EntryType = "Information"}
      }
      Write-EventLog -LogName $EventLogName -Source $EventSource -EventId 1 -EntryType $EntryType -Message $Message
    }
    $msg = ""
  } # end try
  catch {
    Throw "Failed to create log entry in: '$LogPath'. The error was: '$_'."
  } # end catch
} # end function Write-Log

#endregion functions

#region LogHeader
Write-Log -Message "Version         : $Version" -NoConsole
Write-Log -Message "User            : $env:UserDomain\$env:UserName" -NoConsole
Write-Log -Message "Command         : $($MyInvocation.Line)" -NoConsole
Write-Log -Message $LogDivider -NoConsole
#endregion LogHeader

Get-UpdateInfo -Article 2641

Set-ModuleStatus Lync
if ($PoolFqdn){
  $services = @(Get-CsService | Where-Object {$_.version -ge 5 -and $_.PoolFqdn -eq $PoolFqdn})
}else{
  $services = @(Get-CsService | Where-Object {$_.version -ge 5})
}
$AllowedServices = # @("ApplicationServer","ArchivingServer","BackupServer","CentralManagement","ConferencingServer","Director","EdgeServer","MediationServer","MonitoringServer","PersistentChatServer","Registrar","UserServer","WacServer","WebServer")

@("ApplicationServer","ArchivingServer","BackupServer","CentralManagement","ConferencingServer","Director","EdgeServer","MediationServer","MonitoringServer","PersistentChatServer","Registrar","UserServer","WebServer")

[string] $ComputerFqdn = [System.Net.Dns]::GetHostByName(($env:ComputerName)).HostName

foreach ($service in $services)  {
  if ($AllowedServices -icontains $service.role) {
    $pools += @($service.PoolFqdn)
  }
}

$pools = $pools | Select-Object -Unique | Sort-Object
Write-Output "`n"
foreach ($Fqdn in $pools)  {
  $computers = @((Get-CsPool $Fqdn).computers | Sort-Object)

  if ($computers) {
    Write-Host "Pool: $Fqdn `($($computers.count) servers`)`n" -ForegroundColor white
    Write-Log -Message "Pool: $Fqdn `($($computers.count) servers`)" -NoConsole
    foreach ($computer in $computers) {
      if(!(Test-Connection -ComputerName $computer -BufferSize 16 -Count 1 -ErrorAction SilentlyContinue -Quiet)) {
        Write-Host "      $computer (" -ForegroundColor white -NoNewline
        Write-Host "offline" -ForegroundColor red -NoNewline
        Write-Host ")" -ForegroundColor white
        Write-Log -Level Warn -Message "$computer offline" -NoConsole -Indent 1
      } else {
        if ($computer -ne $ComputerFqdn){ # computer is a remote machine
          $version = Invoke-Command -ScriptBlock {
            $filter = @("Debugging Tools","Resource Kit Tools","Best Practices Analyzer","Meeting Room Portal")
            [regex]$regex = '(' + (($filter | foreach {[regex]::escape($_)}) –join "|") + ')'
          @((Get-ChildItem "HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall" | ForEach-Object {Get-ItemProperty $_.pspath} | Where-Object {($_.DisplayName -imatch "Microsoft Lync Server" -and $_.DisplayName -inotmatch $regex) -or ($_.DisplayName -imatch "Microsoft Office Web Apps Server 2013") -or ($_.DisplayName -imatch "Skype for Business Server" -and $_.DisplayName -inotmatch $regex)}) | Sort-Object DisplayVersion -Descending | Select-Object DisplayVersion,DisplayName -First 1)} -ComputerName $computer -ErrorAction SilentlyContinue -WarningAction SilentlyContinue
          $version = $version.DisplayVersion
        }else{ # computer is a local machine (which sometimes has issues with PSRemoting to/from itself)
          # New regex query to filter out non critical apps
          $filter = @("Debugging Tools","Resource Kit Tools","Best Practices Analyzer","Meeting Room Portal")
          [regex]$regex = '(' + (($filter |foreach {[regex]::escape($_)}) –join "|") + ')'
          $version = (Get-ChildItem "HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall" | ForEach-Object {Get-ItemProperty $_.pspath} | Where-Object {($_.DisplayName -imatch "Microsoft Lync Server" -and $_.DisplayName -inotmatch $regex) -or ($_.DisplayName -imatch "Microsoft Office Web Apps Server 2013") -or ($_.DisplayName -imatch "Skype for Business Server" -and $_.DisplayName -inotmatch $regex)}) | Sort-Object DisplayVersion -Descending | Select-Object DisplayVersion,DisplayName -First 1
          $version = $version.DisplayVersion
        }
        Write-Host "      $($computer.split('.')[0]) (" -ForegroundColor white -NoNewline
        if (!$version) {
          Write-Host "PSRemoting failure" -ForegroundColor red -NoNewline
          Write-Host ")" -ForegroundColor white
          Write-Log -Level Warn -Message "$computer - PSRemoting failure" -NoConsole -Indent 1
        } else {
          # Set a default value in case it doesn't match anything below
          $FriendlyResult="Unknown Version"

          [bool] $NotUpToDate = $false

          switch ([string] $version) {
            # Lync Server 2010 updates - https://support.microsoft.com/en-us/kb/2493736
            "4.0.7577.0" {$FriendlyResult = "Lync Server 2010 RTM - 10/2010";$NotUpToDate = $true}
            "4.0.7577.108" {$FriendlyResult = "Lync Server 2010 CU1 - 01/2011";$NotUpToDate = $true}
            "4.0.7577.118" {$FriendlyResult = "Lync Server 2010 post-CU1 - 02/2011";$NotUpToDate = $true}
            "4.0.7577.137" {$FriendlyResult = "Lync Server 2010 CU2 - 04/2011";$NotUpToDate = $true}
            "4.0.7577.166" {$FriendlyResult = "Lync Server 2010 CU3 - 07/2011";$NotUpToDate = $true}
            "4.0.7577.183" {$FriendlyResult = "Lync Server 2010 CU4 - 11/2011";$NotUpToDate = $true}
            "4.0.7577.184" {$FriendlyResult = "Lync Server 2010 post-CU4 - 12/2011";$NotUpToDate = $true}
            "4.0.7577.190" {$FriendlyResult = "Lync Server 2010 CU5 - 02/2012";$NotUpToDate = $true}
            "4.0.7577.197" {$FriendlyResult = "Lync Server 2010 post-CU5 - 03/2012";$NotUpToDate = $true}
            "4.0.7577.199" {$FriendlyResult = "Lync Server 2010 CU6 - 06/2012";$NotUpToDate = $true}
            "4.0.7577.203" {$FriendlyResult = "Lync Server 2010 CU7 - 10/2012";$NotUpToDate = $true}
            "4.0.7577.205" {$FriendlyResult = "Lync Server 2010 post-CU7 - 10/2012";$NotUpToDate = $true}
            "4.0.7577.216" {$FriendlyResult = "Lync Server 2010 CU8 - 03/2013";$NotUpToDate = $true}
            "4.0.7577.217" {$FriendlyResult = "Lync Server 2010 CU9 - 07/2013";$NotUpToDate = $true}
            "4.0.7577.223" {$FriendlyResult = "Lync Server 2010 CU10 - 10/2013";$NotUpToDate = $true}
            "4.0.7577.225" {$FriendlyResult = "Lync Server 2010 CU11 - 01/2014";$NotUpToDate = $true}
            "4.0.7577.230" {$FriendlyResult = "Lync Server 2010 CU12 - 04/2014";$NotUpToDate = $true}
            "4.0.7577.707" {$FriendlyResult = "Lync Server 2010 CU13 - 09/2014";$NotUpToDate = $true}
            # "4.0.7577.708" {$FriendlyResult = "Lync Server 2010 CU14 - 12/2014";$NotUpToDate = $true}
            "4.0.7577.709" {$FriendlyResult = "Lync Server 2010 CU14 - 12/2014";$NotUpToDate = $true}
            "4.0.7577.710" {$FriendlyResult = "Lync Server 2010 CU15 - 12/2014";$NotUpToDate = $true}
            "4.0.7577.713" {$FriendlyResult = "Lync Server 2010 CU16 - 05/2015";$NotUpToDate = $true}
            "4.0.7577.726" {$FriendlyResult = "Lync Server 2010 - 04/2016";$NotUpToDate = $true}
            "4.0.7577.728" {$FriendlyResult = "Lync Server 2010 - 04/2016"}

            # Lync Server 2013 updates - https://www.microsoft.com/en-us/download/details.aspx?id=36820 (KB 2809243)
            "5.0.8308.0" {$FriendlyResult = "Lync Server 2013 RTM";$NotUpToDate = $true}
            "5.0.8308.291" {$FriendlyResult = "Lync Server 2013 CU1 - 02/2013";$NotUpToDate = $true}
            "5.0.8308.420" {$FriendlyResult = "Lync Server 2013 CU2 - 07/2013";$NotUpToDate = $true}
            "5.0.8308.556" {$FriendlyResult = "Lync Server 2013 CU3 - 10/2013";$NotUpToDate = $true}
            "5.0.8308.577" {$FriendlyResult = "Lync Server 2013 CU4 - 01/2014";$NotUpToDate = $true}
            "5.0.8308.603" {$FriendlyResult = "Lync Server 2013 Post CU4";$NotUpToDate = $true}
            "5.0.8308.738" {$FriendlyResult = "Lync Server 2013 CU5 - 08/2014";$NotUpToDate = $true}
            "5.0.8308.803" {$FriendlyResult = "Lync Server 2013 Post CU5";$NotUpToDate = $true}
            "5.0.8308.815" {$FriendlyResult = "Lync Server 2013 CU6 - 09/2014";$NotUpToDate = $true}
            "5.0.8308.831" {$FriendlyResult = "Lync Server 2013 CU7 - 10/2014";$NotUpToDate = $true}
            "5.0.8308.834" {$FriendlyResult = "Lync Server 2013 CU8 - 11/2014";$NotUpToDate = $true}
            "5.0.8308.857" {$FriendlyResult = "Lync Server 2013 CU9 - 12/2014";$NotUpToDate = $true}
            "5.0.8308.866" {$FriendlyResult = "Lync Server 2013 CU10 - 12/31/2014";$NotUpToDate = $true}
            "5.0.8308.871" {$FriendlyResult = "Lync Server 2013 CU11 - 02/19/2015";$NotUpToDate = $true}
            "5.0.8308.872" {$FriendlyResult = "Lync Server 2013 CU11 - 02/19/2015 - updated";$NotUpToDate = $true}
            "5.0.8308.887" {$FriendlyResult = "Lync Server 2013 CU12 - 05/1/2015";$NotUpToDate = $true}
            "5.0.8308.920" {$FriendlyResult = "Lync Server 2013 CU13 - 07/10/2015";$NotUpToDate = $true}
            "5.0.8308.927" {$FriendlyResult = "Lync Server 2013 CU14 - 09/07/2015";$NotUpToDate = $true}
            "5.0.8308.933" {$FriendlyResult = "Lync Server 2013 CU15 - 10/01/2015";$NotUpToDate = $true}
            "5.0.8308.941" {$FriendlyResult = "Lync Server 2013 CU16 - 12/14/2015";$NotUpToDate = $true}
            "5.0.8308.945" {$FriendlyResult = "Lync Server 2013 CU17 - 01/03/2016";$NotUpToDate = $true}
            "5.0.8308.949" {$FriendlyResult = "Lync Server 2013 - 04/15/2016";$NotUpToDate = $true}
            "5.0.8308.956" {$FriendlyResult = "Lync Server 2013 - 05/12/2016";$NotUpToDate = $true}
            "5.0.8308.965" {$FriendlyResult = "Lync Server 2013 - 08/23/2016"}

            # Skype for Business Server 2015 updates - https://www.microsoft.com/en-us/download/details.aspx?id=47690 (KB 3061064)
            "6.0.9319.0" {$FriendlyResult = "Skype for Business Server 2015 RTM";$NotUpToDate = $true}
            "6.0.9319.42" {$FriendlyResult = "Skype for Business Server 2015 - 05/15/2015 [TAP]";$NotUpToDate = $true}
            "6.0.9319.45" {$FriendlyResult = "Skype for Business Server 2015 - 06/19/2015 [TAP]";$NotUpToDate = $true}
            "6.0.9319.48" {$FriendlyResult = "Skype for Business Server 2015 - 05/24/2015 [TAP]";$NotUpToDate = $true}
            "6.0.9319.55" {$FriendlyResult = "Skype for Business Server 2015 - 06/16/2015";$NotUpToDate = $true}
            "6.0.9319.72" {$FriendlyResult = "Skype for Business Server 2015 - 09/04/2015";$NotUpToDate = $true}
            "6.0.9319.88" {$FriendlyResult = "Skype for Business Server 2015 - 09/26/2015";$NotUpToDate = $true}
            "6.0.9319.102" {$FriendlyResult = "Skype for Business Server 2015 - 11/17/2015";$NotUpToDate = $true}
            "6.0.9319.235" {$FriendlyResult = "Skype for Business Server 2015 CU2 - 03/18/2016";$NotUpToDate = $true}
            "6.0.9319.259" {$FriendlyResult = "Skype for Business Server 2015 - 06/27/2016";$NotUpToDate = $true}
            "6.0.9319.272" {$FriendlyResult = "Skype for Business Server 2015 - CU4 11/04/2016"} # preview
          }
          Write-Host $version -ForegroundColor yellow -NoNewline
          Write-Log -Message "$computer - $version" -NoConsole - Indent 1
          if ($FriendlyResult){
            Write-Host " [" -ForegroundColor white -NoNewline
            if (-not $NotUpToDate -and ($FriendlyResult -ne "Unknown Version")){
              Write-Host "$FriendlyResult" -ForegroundColor green -NoNewline
            }else{
              Write-Host "$FriendlyResult" -ForegroundColor red -NoNewline
            }
            Write-Host "]" -ForegroundColor white -NoNewline
          }
          Write-Host ")" -ForegroundColor white
        }
      }
    }
  }else{
    Write-Output "No computers found"
  }
  Write-Host
}


# SIG # Begin signature block
# MIIcdgYJKoZIhvcNAQcCoIIcZzCCHGMCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQURvPlcdy++JjCbcvWUmcMRa33
# u2egghelMIIFLjCCBBagAwIBAgIQA6cFj2F2VqaerCviGAu9jTANBgkqhkiG9w0B
# AQsFADByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFz
# c3VyZWQgSUQgQ29kZSBTaWduaW5nIENBMB4XDTE2MTAxMjAwMDAwMFoXDTE3MTIy
# MTEyMDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBT
# dGVybGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYD
# VQQDExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
# CgKCAQEAvVNbtzSVPbpRQfYrhU0lPzCnmYKS2DFpgrL8gI8qUsyojdQJRjF8q5YQ
# uIi92Uwl/hxWcvCf0dU/EbzRZudOr8tsbkLA6x1R7O5Fsd6Q2fwMN4YmhFDKUcxf
# Gjy1byOdEhSQZqUc4xp6T+prcIi1j4EDSp5R0TQSRiKUK5HoqanhRyGfqJjkEOep
# KB9soxNVkB+jTXsIjX1AeuUJZS/m+1rzWf75laoid+rna7KQtJO+DkVNSa96tTH2
# 1Fs/C78UzWBLA5I2gQAgEA3alkMzB7rIr34Grt251O+yMSyCcvjb70AVcKohoS7n
# 2Zk522EK3jJw3F8euSWMbXNegJtPjwIDAQABo4IBxTCCAcEwHwYDVR0jBBgwFoAU
# WsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYDVR0OBBYEFE+s6yTjN7ocNskZ2gxbJ3Ul
# xBo6MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzB3BgNVHR8E
# cDBuMDWgM6Axhi9odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vc2hhMi1hc3N1cmVk
# LWNzLWcxLmNybDA1oDOgMYYvaHR0cDovL2NybDQuZGlnaWNlcnQuY29tL3NoYTIt
# YXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0gBEUwQzA3BglghkgBhv1sAwEwKjAoBggr
# BgEFBQcCARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAIBgZngQwBBAEw
# gYQGCCsGAQUFBwEBBHgwdjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tME4GCCsGAQUFBzAChkJodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRTSEEyQXNzdXJlZElEQ29kZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/
# BAIwADANBgkqhkiG9w0BAQsFAAOCAQEAL3G7W2y46ZvBae5m6LR+rZu1yA1kkmk5
# 55rTY9lA9xCd0jrhY3cPzwKxGFy82surJeWDWbDIao7wqGXIfDrDgFahdM63btdl
# LWJacdfrM7l+82Zx22R30P/K5sVmk1UXr9eyOPzrYMiemUVru3xMNSZ4Md7lHrfb
# CODsEGjs+zWo4eqXWcnnOCxMd1cpr4PvKPonUhMAhCyFDD5TDzZDQIbHBDIvfb8j
# rg/cvZ2PArf5QZpLp5CiSGnGpC+O9rZhYbsOmcJVFMWMM0pMEw46T9HXHxHMfDC1
# oYTp8Xyc/JxiQ+084oNoGng4esFZIb18cJFXDfR0f4ie0pJTMvanqjCCBTAwggQY
# oAMCAQICEAQJGBtf1btmdVNDtW+VUAgwDQYJKoZIhvcNAQELBQAwZTELMAkGA1UE
# BhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2lj
# ZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4X
# DTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEyMDAwMFowcjELMAkGA1UEBhMCVVMxFTAT
# BgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEx
# MC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElEIENvZGUgU2lnbmluZyBD
# QTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAPjTsxx/DhGvZ3cH0wsx
# SRnP0PtFmbE620T1f+Wondsy13Hqdp0FLreP+pJDwKX5idQ3Gde2qvCchqXYJawO
# eSg6funRZ9PG+yknx9N7I5TkkSOWkHeC+aGEI2YSVDNQdLEoJrskacLCUvIUZ4qJ
# RdQtoaPpiCwgla4cSocI3wz14k1gGL6qxLKucDFmM3E+rHCiq85/6XzLkqHlOzEc
# z+ryCuRXu0q16XTmK/5sy350OTYNkO/ktU6kqepqCquE86xnTrXE94zRICUj6whk
# PlKWwfIPEvTFjg/BougsUfdzvL2FsWKDc0GCB+Q4i2pzINAPZHM8np+mM6n9Gd8l
# k9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB/wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQD
# AgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMHkGCCsGAQUFBwEBBG0wazAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMGCCsGAQUFBzAChjdodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3J0MIGBBgNVHR8EejB4MDqgOKA2hjRodHRwOi8vY3JsNC5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsMDqgOKA2hjRodHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsME8GA1UdIARI
# MEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxodHRwczovL3d3dy5kaWdp
# Y2VydC5jb20vQ1BTMAoGCGCGSAGG/WwDMB0GA1UdDgQWBBRaxLl7KgqjpepxA8Bg
# +S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzANBgkqhkiG
# 9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgsfCUpdqgdXRwtOhrE7zBh134LYP3DPQ/E
# r4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJeJIFOEKTuP3GOYw4TS63XX0R58zYUBor3
# nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbHJyqhKSgaOnEoAjwukaPAJRHinBRHoXpo
# aK+bp1wgXNlxsQyPu6j4xRJon89Ay0BEpRPw5mQMJQhCMrI2iiQC/i9yfhzXSUWW
# 6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDOmTFjPQgaGLOBm0/GkxAG/AeB+ova+YJJ
# 92JuoVP6EpQYhS6SkepobEQysmah5xikmmRR7zCCBmowggVSoAMCAQICEAMBmgI6
# /1ixa9bV6uYX8GYwDQYJKoZIhvcNAQEFBQAwYjELMAkGA1UEBhMCVVMxFTATBgNV
# BAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8G
# A1UEAxMYRGlnaUNlcnQgQXNzdXJlZCBJRCBDQS0xMB4XDTE0MTAyMjAwMDAwMFoX
# DTI0MTAyMjAwMDAwMFowRzELMAkGA1UEBhMCVVMxETAPBgNVBAoTCERpZ2lDZXJ0
# MSUwIwYDVQQDExxEaWdpQ2VydCBUaW1lc3RhbXAgUmVzcG9uZGVyMIIBIjANBgkq
# hkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAo2Rd/Hyz4II14OD2xirmSXU7zG7gU6mf
# H2RZ5nxrf2uMnVX4kuOe1VpjWwJJUNmDzm9m7t3LhelfpfnUh3SIRDsZyeX1kZ/G
# FDmsJOqoSyyRicxeKPRktlC39RKzc5YKZ6O+YZ+u8/0SeHUOplsU/UUjjoZEVX0Y
# hgWMVYd5SEb3yg6Np95OX+Koti1ZAmGIYXIYaLm4fO7m5zQvMXeBMB+7NgGN7yfj
# 95rwTDFkjePr+hmHqH7P7IwMNlt6wXq4eMfJBi5GEMiN6ARg27xzdPpO2P6qQPGy
# znBGg+naQKFZOtkVCVeZVjCT88lhzNAIzGvsYkKRrALA76TwiRGPdwIDAQABo4ID
# NTCCAzEwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAww
# CgYIKwYBBQUHAwgwggG/BgNVHSAEggG2MIIBsjCCAaEGCWCGSAGG/WwHATCCAZIw
# KAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMwggFkBggr
# BgEFBQcCAjCCAVYeggFSAEEAbgB5ACAAdQBzAGUAIABvAGYAIAB0AGgAaQBzACAA
# QwBlAHIAdABpAGYAaQBjAGEAdABlACAAYwBvAG4AcwB0AGkAdAB1AHQAZQBzACAA
# YQBjAGMAZQBwAHQAYQBuAGMAZQAgAG8AZgAgAHQAaABlACAARABpAGcAaQBDAGUA
# cgB0ACAAQwBQAC8AQwBQAFMAIABhAG4AZAAgAHQAaABlACAAUgBlAGwAeQBpAG4A
# ZwAgAFAAYQByAHQAeQAgAEEAZwByAGUAZQBtAGUAbgB0ACAAdwBoAGkAYwBoACAA
# bABpAG0AaQB0ACAAbABpAGEAYgBpAGwAaQB0AHkAIABhAG4AZAAgAGEAcgBlACAA
# aQBuAGMAbwByAHAAbwByAGEAdABlAGQAIABoAGUAcgBlAGkAbgAgAGIAeQAgAHIA
# ZQBmAGUAcgBlAG4AYwBlAC4wCwYJYIZIAYb9bAMVMB8GA1UdIwQYMBaAFBUAEisT
# mLKZB+0e36K+Vw0rZwLNMB0GA1UdDgQWBBRhWk0ktkkynUoqeRqDS/QeicHKfTB9
# BgNVHR8EdjB0MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNl
# cnRBc3N1cmVkSURDQS0xLmNybDA4oDagNIYyaHR0cDovL2NybDQuZGlnaWNlcnQu
# Y29tL0RpZ2lDZXJ0QXNzdXJlZElEQ0EtMS5jcmwwdwYIKwYBBQUHAQEEazBpMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQQYIKwYBBQUHMAKG
# NWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRENB
# LTEuY3J0MA0GCSqGSIb3DQEBBQUAA4IBAQCdJX4bM02yJoFcm4bOIyAPgIfliP//
# sdRqLDHtOhcZcRfNqRu8WhY5AJ3jbITkWkD73gYBjDf6m7GdJH7+IKRXrVu3mrBg
# JuppVyFdNC8fcbCDlBkFazWQEKB7l8f2P+fiEUGmvWLZ8Cc9OB0obzpSCfDscGLT
# Ykuw4HOmksDTjjHYL+NtFxMG7uQDthSr849Dp3GdId0UyhVdkkHa+Q+B0Zl0DSbE
# Dn8btfWg8cZ3BigV6diT5VUW8LsKqxzbXEgnZsijiwoc5ZXarsQuWaBh3drzbaJh
# 6YoLbewSGL33VVRAA5Ira8JRwgpIr7DUbuD0FAo6G+OPPcqvao173NhEMIIGzTCC
# BbWgAwIBAgIQBv35A5YDreoACus/J7u6GzANBgkqhkiG9w0BAQUFADBlMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJvb3QgQ0Ew
# HhcNMDYxMTEwMDAwMDAwWhcNMjExMTEwMDAwMDAwWjBiMQswCQYDVQQGEwJVUzEV
# MBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29t
# MSEwHwYDVQQDExhEaWdpQ2VydCBBc3N1cmVkIElEIENBLTEwggEiMA0GCSqGSIb3
# DQEBAQUAA4IBDwAwggEKAoIBAQDogi2Z+crCQpWlgHNAcNKeVlRcqcTSQQaPyTP8
# TUWRXIGf7Syc+BZZ3561JBXCmLm0d0ncicQK2q/LXmvtrbBxMevPOkAMRk2T7It6
# NggDqww0/hhJgv7HxzFIgHweog+SDlDJxofrNj/YMMP/pvf7os1vcyP+rFYFkPAy
# IRaJxnCI+QWXfaPHQ90C6Ds97bFBo+0/vtuVSMTuHrPyvAwrmdDGXRJCgeGDboJz
# PyZLFJCuWWYKxI2+0s4Grq2Eb0iEm09AufFM8q+Y+/bOQF1c9qjxL6/siSLyaxhl
# scFzrdfx2M8eCnRcQrhofrfVdwonVnwPYqQ/MhRglf0HBKIJAgMBAAGjggN6MIID
# djAOBgNVHQ8BAf8EBAMCAYYwOwYDVR0lBDQwMgYIKwYBBQUHAwEGCCsGAQUFBwMC
# BggrBgEFBQcDAwYIKwYBBQUHAwQGCCsGAQUFBwMIMIIB0gYDVR0gBIIByTCCAcUw
# ggG0BgpghkgBhv1sAAEEMIIBpDA6BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdp
# Y2VydC5jb20vc3NsLWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIB
# Vh6CAVIAQQBuAHkAIAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkA
# ZgBpAGMAYQB0AGUAIABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAA
# dABhAG4AYwBlACAAbwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAA
# LwBDAFAAUwAgAGEAbgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIA
# dAB5ACAAQQBnAHIAZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQA
# IABsAGkAYQBiAGkAbABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIA
# cABvAHIAYQB0AGUAZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUA
# bgBjAGUALjALBglghkgBhv1sAxUwEgYDVR0TAQH/BAgwBgEB/wIBADB5BggrBgEF
# BQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBD
# BggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0
# QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHoweDA6oDigNoY0aHR0cDovL2Ny
# bDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDA6oDig
# NoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9v
# dENBLmNybDAdBgNVHQ4EFgQUFQASKxOYspkH7R7for5XDStnAs0wHwYDVR0jBBgw
# FoAUReuir/SSy4IxLVGLp6chnfNtyA8wDQYJKoZIhvcNAQEFBQADggEBAEZQPsm3
# KCSnOB22WymvUs9S6TFHq1Zce9UNC0Gz7+x1H3Q48rJcYaKclcNQ5IK5I9G6OoZy
# rTh4rHVdFxc0ckeFlFbR67s2hHfMJKXzBBlVqefj56tizfuLLZDCwNK1lL1eT7EF
# 0g49GqkUW6aGMWKoqDPkmzmnxPXOHXh2lCVz5Cqrz5x2S+1fwksW5EtwTACJHvzF
# ebxMElf+X+EevAJdqP77BzhPDcZdkbkPZ0XN1oPt55INjbFpjE/7WeAjD9KqrgB8
# 7pxCDs+R1ye3Fu4Pw718CqDuLAhVhSK46xgaTfwqIa1JMYNHlXdx3LEbS0scEJx3
# FMGdTy9alQgpECYxggQ7MIIENwIBATCBhjByMQswCQYDVQQGEwJVUzEVMBMGA1UE
# ChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYD
# VQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29kZSBTaWduaW5nIENBAhAD
# pwWPYXZWpp6sK+IYC72NMAkGBSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKAC
# gAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsx
# DjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBRUHgv9j9szmqkLOt53uiQs
# IR1zeTANBgkqhkiG9w0BAQEFAASCAQAsnPySB5ZUuhrowtB4qOYGYNGdJT433SOD
# 9tMEsrSJ1lF/0kj/6o6iNj+3Bd8yJjd0rj5LoOvyGTx5G2Vy0fkQP46o8IZGSWmC
# HuJLbIqq3dTwXlFJ+S91ENKJ7YKqGtBxEzT8jY5epHaDLBWuGiT8gs5Gl/aWW6AU
# 995kaUiZY/JlhqiV2y67hF4xJracrgEeXgg+86j/TaIo8qdRE64GgJGVRfiLxmIj
# JFS9V80hq643uKfllGvfIteaUmRlHfNa19ML6uDGsA2md0pBiAWxG4eviif03mLL
# 7I9AUC5pxQqB4FQGYpAAWuKG1mgLrp+CGPNo2V2lKnG4osfMDXzioYICDzCCAgsG
# CSqGSIb3DQEJBjGCAfwwggH4AgEBMHYwYjELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UE
# AxMYRGlnaUNlcnQgQXNzdXJlZCBJRCBDQS0xAhADAZoCOv9YsWvW1ermF/BmMAkG
# BSsOAwIaBQCgXTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJ
# BTEPFw0xNjExMTIyMzMyNDlaMCMGCSqGSIb3DQEJBDEWBBQ3Xo9UZ6WzHVgwGD7E
# /sDN4V2j3DANBgkqhkiG9w0BAQEFAASCAQAxrFQk/WLFz4mivxnJkTxsO/9SyDUw
# hGIvBAlqGnsoaMZLzuMKnh5y4oVr0tpZYpE7gsE+zOjjRzoA3/18RXgtpCPU3+qx
# 2Dz+4N07vIuRvq3m+PAxa1SJUzOmgL3Ur3MrKSmle2ViR23de1VMFCjEAP/H40o/
# nQOqZins8w7Du5znhj1wrKRk+3UpG6R0CGxF7LHfbofh0LLNjg+rp6ke12Bys2xA
# Z341+RVLWn4tVWSEsLkKuRN7jeeiyXGcDuKVJ8AziWGayFZlH4/0lXJZ6PcuX2Mt
# PrtkrBiCcrXZ5JfF4GJLVy0qwnb4Zqo8PQLNbeP86HFiNUYz60WgLEvd
# SIG # End signature block
